% Text Mining with MATLAB 2nd Edition
% www.http://www.textmininglab.net/

% CHAPTER 12: Document Search

disp('Example 12.1 -----------------------------------------------------');
clear; load datasets_ch12a                                          %(12.1)
whos          
pause;

% GENERATES Figure 12.1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figtitle = 'Distribution of documents along books and chapters';
hf=figure(1); set(hf,'Color',[1 1 1],'Name',figtitle);
subplot(3,1,1); hist(labels(labels<2000),1053-1000); 
temp = axis; axis([1001,1053,temp(3:4)]); ylabel('Oliver Twist');
subplot(3,1,2); hist(labels((labels>=2000)&(labels<3000)),2126-2000);
temp = axis; axis([2001,2126,temp(3:4)]); ylabel('Don Quixote');
subplot(3,1,3); hist(labels(labels>=3000),3061-3000);
temp = axis; axis([3001,3061,temp(3:4)]);ylabel('Pride and Prejudice');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 12.2 -----------------------------------------------------');
% creates indexes for the query keywords                           %(12.2a)
idx1 = vocab=="courage";
idx2 = vocab=="lions";
% performs the search 	                                           %(12.2b)
tfmtx = bowmodel.Counts;
bsearch_and = find((tfmtx(:,idx1)>0)&(tfmtx(:,idx2)>0)); 
% displays the categories of the retrieved documents 	           %(12.2c)
labels(bsearch_and)'
pause;

disp('Example 12.3 -----------------------------------------------------');
sum(labels(bsearch_and)==2069)/length(bsearch_and)*100              %(12.3)
pause;

disp('Example 12.4 -----------------------------------------------------');
sum(labels==2069)                                                   %(12.4)
pause;

disp('Example 12.5 -----------------------------------------------------');
sum(labels(bsearch_and)==2069)/sum(labels==2069)*100                %(12.5)
pause;

disp('Example 12.6 -----------------------------------------------------');
%Not a script
pause;

disp('Example 12.7 -----------------------------------------------------');
%Not a script
pause;

disp('Example 12.8 -----------------------------------------------------');
% generates an indicator vector for the selected documents	       %(12.8a)
selected = zeros(ndocs,1);
selected(bsearch_and) = 1;
% generates an indicator vector for the target category            %(12.8b)
targets = double(labels==2069);
% computes the cardinality of the TP, TN, FP, FN subsets	       %(12.8c)
true_positives = sum((selected==1)&(targets==1));
true_negatives = sum((selected==0)&(targets==0));
false_positives = sum((selected==1)&(targets==0));
false_negatives = sum((selected==0)&(targets==1));
% constructs and displays the confusion matrix 	                   %(12.8d)
temp = confusionmat(selected,targets,'ORDER',[1,0]);
cmtx = array2table(temp,'VariableNames',["Category","Non-Category"]);
cmtx.Selections = ["Selected";"Non-Selected"];
disp(cmtx)
% computes and displays the precision, recall and F-score 	       %(12.8e)
precision = true_positives/(true_positives+false_positives)*100;
recall = true_positives/(true_positives+false_negatives)*100;
fscore = 2*precision*recall/(precision+recall);
scores = [precision,recall,fscore];
fprintf('Precision = %6.2f   Recall = %6.2f   F-score = %6.2f\n',scores)
pause;

disp('Example 12.9 ----------------------------------------------------');
%Not a script
pause;

disp('Example 12.10 ----------------------------------------------------');
% searches by using the OR operator with keyword matches 	      %(12.10a)
bsearch_or = find((tfmtx(:,idx1)>0)|(tfmtx(:,idx2)>0)); 
% counts the number of documents retrieved                        %(12.10b)
disp(length(bsearch_or))
% counts the number of documents in the correct category          %(12.10c)
disp(sum(labels(bsearch_or)==2069))
pause;

disp('Example 12.11 ----------------------------------------------------');
% generates an indicator vector for the selected documents	      %(12.11a)
selected = zeros(ndocs,1);
selected(bsearch_or) = 1;
% computes and displays precision, recall and F-score	          %(12.11b)
scores = getscores(selected,targets);
fprintf('Precision = %6.2f   Recall = %6.2f   F-score = %6.2f\n',scores)
pause;

disp('Example 12.12 ----------------------------------------------------');
load datasets_ch12b % loads the queries and categories            %(12.12a)
whos queries qrylbls nqrys
% gets the TF matrix (word counts) for the queries	              %(12.12b)
qrytfmtx = encode(bowmodel,queries);
pause;

disp('Example 12.13 ----------------------------------------------------');
bsearch_or_selected = zeros(ndocs,nqrys); 	                       %(12.13)
bsearch_and_selected = zeros(ndocs,nqrys); 
for q=1:nqrys
    keywordmask = full(qrytfmtx(q,:)>0);
    for d=1:ndocs 
        keywordsindoc = full(tfmtx(d,keywordmask)>0);
        bsearch_and_selected(d,q) = prod(keywordsindoc+0)>0;
        bsearch_or_selected(d,q) = sum(keywordsindoc)>0;
    end
end
pause;

disp('Example 12.14 ----------------------------------------------------');
% computes the scores for each query and search strategy  	      %(12.14a)
for k=1:nqrys
    targets = labels==qrylbls(k); % sets the target category vector 
    % AND-based search strategy: 
    selected = bsearch_and_selected(:,k);
    scores_and(k,:) = getscores(selected,targets);
    ndocs_and(k) = sum(selected);   
    % OR-based search strategy: 
    selected = bsearch_or_selected(:,k); 
    scores_or(k,:) = getscores(selected,targets);
    ndocs_or(k) = sum(selected); 
end
% prints the report for the AND search strategy	                  %(12.14b)
variables = ["Selections","Precision","Recall","F-score"];
querynumber = "Q"+string(1:nqrys)';
tbl_and = array2table([ndocs_and',scores_and],'VariableNames',variables);
disp(addvars(tbl_and,querynumber,'NewVariableNames',"Query",'Before',1))
% prints the report for the OR search strategy	                  %(12.14c)
tbl_or = array2table([ndocs_or',scores_or],'VariableNames',variables);
disp(addvars(tbl_or,querynumber,'NewVariableNames',"Query",'Before',1))
pause;

disp('Example 12.15 ----------------------------------------------------');
% computes score averages    	                                   %(12.15)
or_averages = mean(scores_or);
and_averages = mean(scores_and);
pause;

% GENERATES FIGURE 12.3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hf = figure(3); 
figname = 'Comparison between AND and OR search strategies';
set(hf,'Color',[1 1 1],'Name',figname);
hs = subplot(1,1,1);
bar([or_averages;and_averages]');
set(hs,'XTickLabel',{'Precision','Recall','F-score'});
legend('Binary Search (OR)','Binary Search (AND)');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 12.16 ----------------------------------------------------');
% computes the TF-IDF matrix for the queries	                   %(12.16)
qrytfidfmtx = encode(bowmodel,queries);
pause;

disp('Example 12.17 ----------------------------------------------------');
% computes cosine similarities and ranks with respect to query 5  %(12.17a)
qrynum = 5; 
vsearch = full(cosineSimilarity(qrytfidfmtx,tfidfmtx));
[~,ranks] = sort(vsearch(qrynum,:),'descend');
% computes the scores at each rank position                       %(12.17b)
for k=1:ndocs
    selected = zeros(ndocs,1); 
    selected(ranks(1:k)) = 1;
    targets = labels==qrylbls(qrynum);
    scores(k,:) = getscores(selected,targets);
end
pause;

% GENERATES Figure 12.4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figtitle = 'Precision, recall and F-score vs. ranking';
hf=figure(4); set(hf,'Color',[1 1 1],'Name',figtitle);
semilogx(1:ndocs,scores(:,1),'.:',1:ndocs,scores(:,2),'.-.',1:ndocs,scores(:,3),'.-');
legend('Precision','Recall','F-score','Location','North'); 
axis([1,ndocs,0,100]);
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 12.18 ----------------------------------------------------');
rankloc = 10; % sets the rank position at 10                      %(12.18a)
% computes the scores for all the queries                         %(12.18b)
for k=1:nqrys 
    [~,ranks] = sort(vsearch(k,:),'descend');
    selected = zeros(ndocs,1); 
    selected(ranks(1:rankloc)) = 1;
    targets = labels==qrylbls(k);
    scores_v10(k,:) = getscores(selected,targets);
end
% computes score averages                                         %(12.18c)
v10_averages = mean(scores_v10);
pause;

% GENERATES FIGURE 12.5 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figname = 'Comparison between vector and binary search strategies';
hf = figure(5); set(hf,'Color',[1 1 1],'Name',figname);
hs = subplot(1,1,1);
bar([or_averages;and_averages;v10_averages]');
set(hs,'XTickLabel',{'Precision','Recall','F-score'});
legend('Binary Search (OR)','Binary Search (AND)','Vector Search @10');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 12.19 ----------------------------------------------------');
qrynum = 5;    	                                                   %(12.19)
[weight,idxs] = sort(qrytfidfmtx(qrynum,:),'descend');
vocab(idxs(weight>0))
pause;

disp('Example 12.20 ----------------------------------------------------');
% recomputes document ranks for query 5 	                      %(12.20a)
[~,ranks] = sort(vsearch(qrynum,:),'descend');
% computes the contribution of the most relevant documents        %(12.20b)
mrv = 2; % number of most relevant samples to be considered 
vplus = sum(tfidfmtx(ranks(1:mrv),:))/mrv;
% computes the contribution of the least relevant documents       %(12.20c)
lrv = 100; % number of least relevant samples to be considered 
vminus = sum(tfidfmtx(ranks(end-lrv+1:end),:))/lrv;
% computes the expanded the query                                 %(12.20d)
a = 0.90; % sets the value of the combination parameter 
newqry = qrytfidfmtx(qrynum,:) +a*vplus -(1-a)*vminus; % expands the query
newqry = newqry.*(newqry>0); % discards resulting negative entries (if any)
pause;

disp('Example 12.21 ----------------------------------------------------');
[~,idxs] = sort(newqry,'descend');    	                       %(12.21)
disp(vocab(idxs(1:15)))
pause;

disp('Example 12.22 ----------------------------------------------------');
% creates the target category vector                              %(12.22a)
targets = labels==qrylbls(qrynum);
% vector-based search for the original query                      %(12.22b)
vsearch = full(cosineSimilarity(qrytfidfmtx(qrynum,:),tfidfmtx)); 
[~,ranks] = sort(vsearch,'descend');
for k=1:50
    selected = zeros(ndocs,1); 
    selected(ranks(1:k)) = 1;
    original(k,:) = getscores(selected,targets);
end
% vector-based search for the expanded query	                  %(12.22c)
vsearch = full(cosineSimilarity(newqry,tfidfmtx));  
[~,ranks] = sort(vsearch,'descend');
for k=1:50
    selected = zeros(ndocs,1); 
    selected(ranks(1:k)) = 1;
    expanded(k,:) = getscores(selected,targets);
end
pause;

disp('Example 12.23 ----------------------------------------------------');
% defines the rank positions to be displayed and tags	          %(12.23a)
ranks = 1:2:30;
for k=1:length(ranks), xlbls(k)= sprintf("@%02d",ranks(k)); end
% initializes the figure                                          %(12.23b)
figname = 'Comparison between original and expanded queries';
hf = figure(6); set(hf,'Color',[1 1 1],'Name',figname);
hs = subplot(3,1,1); % displays the precision bar plot            %(12.23c)
bar([original(ranks,1),expanded(ranks,1)]); 
legend('original','expanded'); 
set(hs,'XTickLabel',xlbls); ylabel('Precision');
hs = subplot(3,1,2); % displays the recall bar plot               %(12.23d)
bar([original(ranks,2),expanded(ranks,2)]);
set(hs,'XTickLabel',xlbls); ylabel('Recall');
hs = subplot(3,1,3); % displays the f-score bar plot              %(12.23e)
bar([original(ranks,3),expanded(ranks,3)]);
set(hs,'XTickLabel',xlbls); ylabel('F-score');
pause;

disp('Example 12.24 ----------------------------------------------------');
%Not a script
pause;

disp('Example 12.25 ----------------------------------------------------');
%Not a script
pause;

disp('Example 12.26 ----------------------------------------------------');
%Not a script
pause;

disp('Example 12.27 ----------------------------------------------------');
% computes BM25 relevance scores                                  %(12.27a)
bmsearch = bm25Similarity(docs,queries);
% gets document rankings and evaluates performance                %(12.27b)
for k=1:nqrys
    targets = labels==qrylbls(k); 
    [~,ranks] = sort(bmsearch(:,k),'descend');
    selected = zeros(ndocs,1); 
    selected(ranks(1:rankloc)) = 1;
    scores_bm25(k,:) = getscores(selected,targets); 
end
% computes score averages                                         %(12.27c)
bm25_averages = mean(scores_bm25);
pause;

disp('Example 12.28 ----------------------------------------------------');
% displays results from BM25 (12.27) and vector search (12.18)     %(12.28)
datavalues = [v10_averages;bm25_averages];
tbl = array2table(datavalues,'VariableNames',variables(2:end));
methodnames = ["Vector @10";"BM25 @10"];
disp(addvars(tbl,methodnames,'NewVariableNames',"Method",'Before',1))
pause;

disp('Example 12.29 ----------------------------------------------------');
% implements BM11 search                                           %(12.29)
bm11search = bm25Similarity(docs,queries,'DocumentLengthScaling',1);
for k=1:nqrys
    targets = labels==qrylbls(k); 
    [~,ranks] = sort(bm11search(:,k),'descend');
    selected = zeros(ndocs,1); selected(ranks(1:rankloc)) = 1;
    scores_bm11(k,:) = getscores(selected,targets); 
end
bm11_averages = mean(scores_bm11);
pause;

disp('Example 12.30 ----------------------------------------------------');
% implements BM15 search                                           %(12.30)
bm15search = bm25Similarity(docs,queries,'DocumentLengthScaling',0);
for k=1:nqrys
    targets = labels==qrylbls(k); 
    [~,ranks] = sort(bm15search(:,k),'descend');
    selected = zeros(ndocs,1); selected(ranks(1:rankloc)) = 1;
    scores_bm15(k,:) = getscores(selected,targets); 
end
bm15_averages = mean(scores_bm15);
pause;

% GENERATES FIGURE 12.7 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figname = 'Comparison among different BM25 variants';
hf = figure(7); set(hf,'Color',[1 1 1],'Name',figname);
hs = subplot(1,1,1);
bar([bm25_averages;bm11_averages;bm15_averages]');
set(hs,'XTickLabel',{'Precision','Recall','F-score'});
legend('BM25 (b=0.75)','BM11 (b=1)','BM15 (b=0)');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% GENRATES THE TABLE IN SECTION 12.4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load datasets_ch12c
fprintf('text segments %d %d\n',length(docs_eng),length(docs_spa));
leng = doclength(docs_eng); lspa = doclength(docs_spa);
fprintf('min segment length %d %d\n',min(leng),min(lspa));
fprintf('max segment length %d %d\n',max(leng),max(lspa));
fprintf('aver segment length %f %f\n',mean(leng),mean(lspa));
fprintf('running wordsh %d %d\n',sum(leng),sum(lspa));
fprintf('vocabulary terms %d %d\n',length(vocab_eng),length(vocab_spa));
esing =full(sum(sum(bowmodel_eng.Counts)==1));
ssing =full(sum(sum(bowmodel_spa.Counts)==1));
fprintf('singleton terms %d %d\n',esing,ssing);
common = length([vocab_spa,vocab_eng])-length(unique([vocab_spa,vocab_eng]));
fprintf('common terms %d %d\n',common,common);
%intersect(vocab_eng,vocab_spa)
pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 12.31 ----------------------------------------------------');
load datasets_ch12c % loads the bilingual collection              %(12.31a)
% gets the total number of words in each text segment             %(12.31b)
nw_spa = doclength(docs_spa); 
nw_eng = doclength(docs_eng);
% computes a linear regression between nw_spa and nw_eng          %(12.31c)
coeff = regress(nw_spa',[nw_eng',ones(length(nw_eng),1)]); 
x_reg = [10,50]; y_reg = coeff(1)*x_reg + coeff(2);
% generates the cross-plot                                        %(12.31d)
hf = figure(8); 
figname ='Cross-plot between English and Spanish segment lengths';
set(hf,'Color',[1 1 1],'Name',figname);
plot(nw_eng,nw_spa,'.',x_reg,y_reg,'-k',x_reg,x_reg,':k');
legend('data samples','regression line','symmetry line',...
       'Location','NorthWest'); 
axis([10,50,10,50]);
xlabel('Length of English text segments');
ylabel('Length of Spanish text segments');
pause;

disp('Example 12.32 ----------------------------------------------------');
% applies vocabulary pruning to the English subset                %(12.32a)
mask = full(sum(bowmodel_eng.Counts>0)>1);  
pvocab_eng = vocab_eng(mask);
ptfidfmtx_eng = tfidfmtx_eng(:,mask);
% applies vocabulary pruning to the Spanish subset                %(12.32b)
mask = full(sum(bowmodel_spa.Counts>0)>1); 
pvocab_spa = vocab_spa(mask);
ptfidfmtx_spa = tfidfmtx_spa(:,mask);
pause;

disp('Example 12.33 ----------------------------------------------------');
% creates and index for the English word "lions"                  %(12.33a)
widx_eng1 = pvocab_eng=="lions";
% gets pointers to the documents in which the word occurs
docs_eng1 = full(ptfidfmtx_eng(:,widx_eng1)>0);    
% derives the Spanish vector of translation candidates            %(12.33b)
% 1. computes the contribution of the most relevant documents
vplus = sum(ptfidfmtx_spa(docs_eng1,:))/sum(docs_eng1);
% 2. computes the contribution of the least relevant documents
vminus = sum(ptfidfmtx_spa(not(docs_eng1),:))/sum(not(docs_eng1));
% 3. computes the complete vector of translation candidates
spav_eng1 = full(vplus-vminus);  
% gets the 5 most representative Spanish terms for "lions"        %(12.33c)
[vals_eng1,ranks] = sort(spav_eng1,'descend');
spaw_eng1 = pvocab_spa(ranks(1:5)) 
pause;

disp('Example 12.34 ----------------------------------------------------');
widx_eng2 = pvocab_eng=="courage";                                %(12.34a)
docs_eng2 = full(ptfidfmtx_eng(:,widx_eng2)>0);                                        
vplus = sum(ptfidfmtx_spa(docs_eng2,:))/sum(docs_eng2);           %(12.34b)
vminus = sum(ptfidfmtx_spa(not(docs_eng2),:))/sum(not(docs_eng2));
spav_eng2 = full(vplus-vminus);  
[vals_eng2,ranks] = sort(spav_eng2,'descend');                    %(12.34c)
spaw_eng2 = pvocab_spa(ranks(1:5))
pause;

disp('Example 12.35 ----------------------------------------------------');
widx_spa1 = pvocab_spa=="leones";                                 %(12.35a)
docs_spa1 = full(ptfidfmtx_spa(:,widx_spa1)>0); 
vplus = sum(ptfidfmtx_eng(docs_spa1,:))/sum(docs_spa1);
vminus = sum(ptfidfmtx_eng(not(docs_spa1),:))/sum(not(docs_spa1));
engv_spa1 = full(vplus-vminus);  
[vals_spa1,ranks] = sort(engv_spa1,'descend');
engw_spa1 = pvocab_eng(ranks(1:5)) 
widx_spa2 = pvocab_spa=="ánimo";                                  %(12.35b)
docs_spa2 = full(ptfidfmtx_spa(:,widx_spa2)>0); 
vplus = sum(ptfidfmtx_eng(docs_spa2,:))/sum(docs_spa2);
vminus = sum(ptfidfmtx_eng(not(docs_spa2),:))/sum(not(docs_spa2));
engv_spa2 = full(vplus-vminus);  
[vals_spa2,ranks] = sort(engv_spa2,'descend');
engw_spa2 = pvocab_eng(ranks(1:5)) 
pause;

disp('Example 12.36 ----------------------------------------------------');
% English segments in which the word "courage" occurs             %(12.36a)
find(docs_eng2)'
% Spanish segments in which the word "ánimo" occurs               %(12.36b)
find(docs_spa2)'
pause;

disp('Example 12.37 ----------------------------------------------------');
disp(rawtext_eng(753))                                            %(12.37a)
disp(rawtext_spa(753))                                            %(12.37b)
pause;

disp('Example 12.38 ----------------------------------------------------');
disp(rawtext_eng(978))                                            %(12.38a)
disp(rawtext_spa(978))                                            %(12.38b)
pause;

disp('Example 12.39 ----------------------------------------------------');
% performs the basic figure settings                              %(12.39a)
figname = 'Bar plots from translation inference examples'; 
hf = figure(9); set(hf,'Color',[1 1 1],'Name',figname); 
% creates the bar plot for the "lions" example (12.33)            %(12.39b)
hs = subplot(2,2,1); barh(vals_eng1(1:5)); axis ij;  
set(hs,'YTickLabel',spaw_eng1); title('Spanish words for "\it{lions}"');
% creates the bar plot for the "courage" example (12.34)          %(12.39c)
hs = subplot(2,2,3); barh(vals_eng2(1:5)); axis ij; 
set(hs,'YTickLabel',spaw_eng2); title('Spanish words for "\it{courage}"');
% creates the bar plot for the "leones" example (12.35a)          %(12.39d)
hs = subplot(2,2,2); barh(vals_spa1(1:5)); axis ij; 
set(hs,'YTickLabel',engw_spa1); title('English words for "\it{leones}"');
% creates the bar plot for the "ánimo" example (12.35b)           %(12.39e)
hs = subplot(2,2,4); barh(vals_spa2(1:5)); axis ij; 
set(hs,'YTickLabel',engw_spa2); title('English words for "\it{ánimo}"');
pause;

disp('Example 12.40 ----------------------------------------------------');
% combines documents that are relevant to both Spanish keywords    %(12.40)
mask = docs_spa1|docs_spa2;
% computes the contribution of the relevant documents 
vplus = sum(ptfidfmtx_eng(mask,:))/sum(mask);
% computes the contribution of the non-relevant documents 
vminus = sum(ptfidfmtx_eng(not(mask),:))/sum(not(mask));
a = 0.2; % sets the value of the combination parameter
xqry = full(a*vplus-(1-a)*vminus); % computes the complete vector
xqry = xqry.*(xqry>0); % discards negative entries (if any) 
pause;

disp('Example 12.41 ----------------------------------------------------');
% intersects the vocabularies and gets indexes for common terms    %(12.41)
[~,index1,index2] = intersect(vocab,pvocab_eng);
converted_qry = zeros(1,length(vocab)); % initializes the new query 
converted_qry(index1) = xqry(index2); % performs the conversion 
pause;

disp('Example 12.42 ----------------------------------------------------');
targets = labels==qrylbls(qrynum);                                 %(12.42)
vsearch = full(cosineSimilarity(converted_qry,tfidfmtx));    
[~,ranks] = sort(vsearch,'descend');
for k=1:50
    selected = zeros(ndocs,1);
    selected(ranks(1:k)) = 1;
    crosslang(k,:) = getscores(selected,targets);
end
pause;

% GENERATES FIGURE 12.10 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ranks = 1:2:30;
for k=1:length(ranks)
    xlbls(k)= sprintf("@%02d",ranks(k));
end
figname = 'Comparison between original and cross-language search';
hf = figure(10); set(hf,'Color',[1 1 1],'Name',figname);
hs = subplot(3,1,1); 
bar([original(ranks,1),crosslang(ranks,1)]);
%temp = axis; axis([temp(1:2),0,100]);
legend('original','cross-language');
set(hs,'XTickLabel',xlbls);
ylabel('Precision');
hs = subplot(3,1,2); 
bar([original(ranks,2),crosslang(ranks,2)]);
%temp = axis; axis([temp(1:2),0,40]);
set(hs,'XTickLabel',xlbls);
ylabel('Recall');
hs = subplot(3,1,3); 
bar([original(ranks,3),crosslang(ranks,3)]);
%temp = axis; axis([temp(1:2),0,60]);
set(hs,'XTickLabel',xlbls);
ylabel('F-score');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 12.43 ----------------------------------------------------');
% computes and displays original and cross-language score averages %(12.43)
datavalues = [mean(original);mean(crosslang)];
tbl = array2table(datavalues,'VariableNames',variables(2:end));
methodnames = ["Original";"Cross-Language"];
disp(addvars(tbl,methodnames,'NewVariableNames',"Method",'Before',1))
pause;


